import random
random.seed(13371337)

def solve(n, l, k):
    parent = [i for i in range(n+1)]
    size = [1 for _ in range(n+1)]
    def find(x: int):
        if parent[x] == x: return x
        parent[x] = find(parent[x])
        return parent[x]
    def merge(u: int, v: int):
        u = find(u)
        v = find(v)
        if u == v: return
        if size[v] > size[u]: u, v = v, u
        parent[v] = u
        size[u] += size[v]
    for Si in l:
        for a,b in zip(Si, Si[1:]):
            merge(a, b)
    return [size[find(x)] for x in k]

testnum = 0
def maketest(n: int, l: list[list[int]], k: list[int]):
    global testnum
    with open(f"../input/input{testnum}.txt", 'w') as f:
        f.write(f"{n} {len(l)} {len(k)}\n")
        assert sum(map(len, l)) <= 200000
        for x in l:
            assert all(0 < y <= n for y in x)
            assert len(set(x)) == len(x)
            f.write(f"{len(x)} " + " ".join(map(str, x)) + "\n")
        for x in k:
            assert 0 < x <= n
            f.write(f"{x}\n")
    ans = solve(n, l, k)
    with open(f"../output/output{testnum}.txt", 'w') as f:
        for x in ans:
            f.write(f"{x}\n")
    print(testnum)
    testnum += 1

def shuffled(l):
    out = list(l)
    random.shuffle(out)
    return out

def randomize_clique(l: list[int]):
    out: list[list[int]] = []
    orig_l = l.copy()
    if len(l) < 2:
        return [] # cliques are 1 by default anyways
    if len(l) < 6:
        return [l]
    for i in range(len(l) // 3):
        thisc = l[:3]
        if l[3:6] == []: break
        nextc = random.choice(l[3:6])
        l = l[3:]
        out.append(thisc + [nextc])
    if l != []: out.append(l + [out[0][0]])
    return out

def gen_rand_graph(size: int, clique_sizes: list[int]):
    l = shuffled(range(1, size+1))
    out: list[list[int]] = []
    for x in clique_sizes:
        out += randomize_clique(l[:x])
        l = l[x:]
    return shuffled(out)

print("# ST: 0")
maketest(10, [[1, 2, 3], [2, 5, 8], [8, 4], [9, 10]], [1])
maketest(10, [[3, 4], [6, 4, 3], [5, 7], [7, 8], [8, 5]], [1, 5, 4])

print("# ST: 5")
maketest(100, [[i, i+1] for i in range(1,81)], [69])
maketest(100, [list(range(1,101)) for _ in range(100)], [7])
maketest(100, gen_rand_graph(100, [10, 20, 20, 50]), [6])
maketest(100, [[i, i%50+1] for i in range(1,60)], [42])

print("# ST: 5")
maketest(100, [[i, i+1] for i in range(1,99)], shuffled(range(1, 101)))
maketest(100, [list(range(1,101)) for _ in range(100)], shuffled(range(1, 101)))
maketest(100, gen_rand_graph(100, [10, 20, 20, 50]), shuffled(range(1, 101)))
maketest(100, [[i, i%50+1] for i in range(1,60)], shuffled(range(1, 101)))

print("# ST: 10")
maketest(1000, [[i, i+1] for i in range(1,999)], shuffled(range(1, 1001)))
maketest(1000, gen_rand_graph(1000, list(range(42, 59)) + [11, 139]), shuffled(range(1, 1001)))
maketest(400, [list(range(1,401)) for _ in range(400)], shuffled(range(1, 401)))
maketest(1000, [[i, i%999+1] for i in range(1, 1000)], shuffled(range(1, 1001)))

print("# ST: 20")
maketest(50000, gen_rand_graph(50000, list(range(9998, 10003))), shuffled(range(1, 50001)))
maketest(100000, gen_rand_graph(100000, list(range(4990, 5010))+[1, 1, 1, 7]), shuffled(range(1, 100001)))
maketest(100000, gen_rand_graph(100000, [9, 11] * 5000), shuffled(range(1, 100001)))
maketest(100000, gen_rand_graph(100000, [4, 6] * 9999 + [1, 9]), shuffled(range(1, 100001)))
maketest(100000, [[i, i%99999+1] for i in range(1, 100000)], shuffled(range(1, 100001)))
maketest(100000, shuffled(shuffled(list(range(i * 100000 // 5 + 1, (i + 1) * 100000 // 5 + 1)) + [((i + 1) * 100000 // 5) % 100000 + 1]) for i in range(5)), shuffled(range(1, 100001)))
inds = shuffled(range(1, 100000 + 1))
maketest(100000, [shuffled(inds[i * 100000 // 50:(i + 1) * 100000 // 50] + [inds[(i + 1) * 100000 // 50 % 100000]]) for i in range(50)], shuffled(range(1, 100001)))
