from pathlib import Path
from random import Random
from sys import stderr

gen = open("GEN", "w")

root = Path("../input")
rng = Random()

DEBUG = False

MAX_N = 10**5
MAX_L = 10**9 - 5

points = [0, 5, 10, 15, 10, 20]

gcount = 0
def group_init():
    global gcount
    p = points[gcount]
    print(f"Group {gcount} ({p} points)")
    print(f"# ST: {p}", file=gen)
    gcount += 1

case_open = False
def case_init(name):
    global case_open
    assert not case_open
    case_open = True
    print(f"  {count:2} {name}")
    print(count, file=gen)
    rng.seed("lv2023" + name)

count = 0
def case(rs):
    global count, case_open
    case_open = False
    with open(root / f"input{count}.txt", "w") as f:
        print(len(rs), file=f)
        for r in rs:
            print(*r, file=f)
    count += 1

def partition(l, mxl=4):
    ls = []
    s = 0
    while s < l:
        ls.append(rng.randint(1, mxl))
        s += ls[-1]
    rng.shuffle(ls)
    while s > l:
        s -= ls.pop()
    if s < l:
        ls.append(l - s)
        rng.shuffle(ls)
    return ls
        
def checkerboard(w, h, p=0.0):
    ws = partition(w)
    hs = partition(h)
    
    rs = []
    ty = 1
    ps = set()
    ph = 0
    for hi, ch in enumerate(hs):
        tx = 1
        pw = 0
        for wi, cw in enumerate(ws):
            if (hi + wi) % 2 == 1:
                if rng.random() < p:
                    # perturb
                    if rng.random() < 0.3:
                        # remove
                        pass
                    elif (hi, wi) not in ps:
                        if pw > 0 and (hi, wi - 1) not in ps and rng.random() < 0.5:
                            # x
                            dx = -rng.randint(1, pw)
                            rs.append((tx + dx, ty, cw, ch))
                            ps.add((hi, wi))
                            ps.add((hi, wi - 1))
                        elif ph > 0 and (hi - 1, wi) not in ps:
                            # y
                            dy = -rng.randint(1, ph)
                            rs.append((tx, ty + dy, cw, ch))
                            ps.add((hi, wi))
                            ps.add((hi - 1, wi))
                    else:
                        rs.append((tx, ty, cw, ch))
                else:
                    rs.append((tx, ty, cw, ch))
            tx += cw
            pw = cw
        ty += ch
        ph = ch
    return rs

def fit(rs, n=MAX_N):
    if DEBUG:
        if len(rs) > n:
            p = (len(rs) / n - 1) * 100
            print(f"    {len(rs)} -> {n}  ({p:.1f}%)")
        else:
            print(f"    {len(rs)} <= {n} !")
    
    rng.shuffle(rs)
    while len(rs) > n:
        rs.pop()
    return rs

def transpose(rs):
    return [(y, x, h, w) for x, y, w, h in rs]

def scale_x(rs, f=None):
    if f is None:
        f = min(MAX_L // (x + w) for x, y, w, h in rs)
    return [((x - 1) * f + 1, y, w * f, h) for x, y, w, h in rs]

def scale(rs, f=None):
    return transpose(scale_x(transpose(scale_x(rs, f)), f))


def dense_line(y, w, mxl=4, p=1.0, sx=1):
    ws = partition(w, mxl)
    cx = sx
    rs = []
    for w in ws:
        if rng.random() < p:
            rs.append((cx, y, w, 1))
        cx += w
    return rs

def dense_rect(h, w, mxl, p, sx=1):
    rs = []
    for y in range(1, h + 1):
        rs += dense_line(y, w, mxl, p, sx)
    return rs

def perturb(r):
    x, y, w, h = r
    if rng.random() < 0.02:
        if w > 1 and rng.random() < 0.5:
            # left-right
            dx = rng.randint(1, min(1000, w - 1))
            w -= dx
            if rng.random() < 0.5:
                x += dx
        elif h > 1:
            # up-down
            dy = rng.randint(1, min(1000, h - 1))
            h -= dy
            if rng.random() < 0.5:
                y += dy
    return (x, y, w, h)

def subdiv(rs, n):
    c = 0
    while len(rs) < n and c < 10**7:
        c += 1
        
        ci = rng.randrange(len(rs))
        x, y, w, h = rs[ci]
        x2 = x1 = x
        y2 = y1 = y
        w2 = w1 = w
        h2 = h1 = h
        if w > 1 and rng.random() < 0.5:
            # vertical split
            w1 = rng.randint(1, w - 1)
            x2 = x1 + w1
            w2 = w - w1
        elif h > 1 and (h > 2 or rng.random() < 0.1):
            # horizontal split
            h1 = rng.randint(1, h - 1)
            y2 = y1 + h1
            h2 = h - h1
        else:
            continue
        rs[ci] = perturb((x1, y1, w1, h1))
        rs.append(perturb((x2, y2, w2, h2)))
    return rs
            
# sample
group_init()
case_init("sample1")
case([(2, 1, 2, 1), (1, 2, 4, 1), (5, 1, 2, 1), (6, 2, 1, 1)])

case_init("sample2")
rs = checkerboard(8, 5, 0.5)
case(rs)


# single line
group_init()

for i, ml in enumerate([2, 3000], 1):
    case_init(f"rand_line{i}")
    rs = []
    px = 0
    while len(rs) < MAX_N:
        l = rng.randint(1, ml)
        if rng.random() < 0.3:
            pass
        else:
            rs.append((px + 1, 1, l, 1))
        px += l
    case(rs)

case_init("dense_line")
rs = scale_x(fit(dense_line(1, MAX_N * 13 // 5)))
case(rs)
    
case_init("alt_line")
rs = []
px = 0
while len(rs) < MAX_N:
    l = rng.randint(1, 3000)
    rs.append((px + 1, 1, l, 1))
    l2 = rng.randint(1, 3000)
    px += l + l2
case(rs)

# n <= 1000
group_init()


case_init(f"small_checker")
rs = scale(fit(checkerboard(135, 135, 0.8), 1000))
case(rs)

for i, (w, h, p) in enumerate([(51, 51, 0.95), (260, 13, 0.8)], 1):
    case_init(f"small_dense_rect{i}")
    rs = scale(fit(dense_rect(h, w, 4, p), 1000))
    if i % 2 == 0:
        rs = transpose(rs)
    case(rs)

for i in range(1, 3):
    case_init(f"small_subdiv{i}")
    rs = scale(subdiv([(1, 1, 10000, 10000)], 1000))
    case(rs)
    
case_init(f"small_sandwich")
rs = fit(dense_line(2, MAX_L - 3, 9 * 10**6 // 5, 0.95), 1000 - 2)
rs.append((MAX_L // 8, 1, MAX_L * 6 // 8, 1))
rs.append((1, 3, MAX_L - 3, 1))
rs = scale(transpose(rs))
case(rs)


# y + h <= 3
# aka two lines
group_init()

cl = 9 * 10**4 // 5

case_init("half_sandwich")
rs = fit(dense_line(1, MAX_L - 3, cl, 0.95), MAX_N - 1)
rs.append((1, 2, MAX_L - 3, 1))
case(rs)

for i, (l1, l2, p1, p2) in enumerate([(cl * 2, cl * 2, 0.93, 0.93), (cl * 3 // 2, cl * 3, 0.995, 0.75)], 1):
    case_init(f"two_dense{i}")
    y1 = 1 + i % 2
    y2 = 1 + 2 - y1
    rs  = dense_line(y1, MAX_L - 3, l1, p1)
    rs += dense_line(y2, MAX_L - 3, l2, p2)
    rs = fit(rs)
    case(rs)

for i, p in enumerate([0.95, 0.85], 1):
    case_init(f"two_mixed{i}")
    rs = []
    ws = partition(MAX_L - 5, MAX_L // 130)
    cx = 2
    block = False
    for w in ws:
        if block:
            rs.append((cx, 1, w, 2))
        else:
            rs += dense_rect(2, w, w // int(200 / p), p, cx)
        cx += w
        block = not block
    rs = fit(rs)
    case(rs)

case_init(f"two_bigblocks")
rs = scale(fit(dense_line(1, MAX_L // 3, MAX_L // MAX_N * 2 // 3, 0.97)), 2)
case(rs)

case_init(f"two_subdiv")
rs = scale_x(subdiv([(1, 1, 10**6, 2), (10**6 + 1, 1, 10**6, 2)], MAX_N))
case(rs)


# y + h <= 50
group_init()

case_init(f"sandwich1")
rs = fit(dense_line(2, MAX_L - 3, 21 * 10**4 // 11, 0.993), MAX_N - 2)
rs.append((MAX_L // 8, 1, MAX_L * 6 // 8, 1))
rs.append((1, 3, MAX_L - 3, 1))
case(rs)

case_init(f"squish_checkers")
rs = scale_x(fit(checkerboard(10 * MAX_N // 24, 49, 0.73)))
case(rs)

case_init(f"dense_stripes1")
rs = fit(dense_rect(49, MAX_L - 3, MAX_L // 1030, 0.99))
case(rs)

case_init("mixed")
rs = []
blocks = []
for _ in range(5000):
    x = rng.randint(1, MAX_L - 6000)
    w = rng.randint(1, 50)
    y1 = rng.randint(1, 48)
    y2 = rng.randint(1, 48)
    if y1 > y2:
        y1, y2 = y2, y1
    y = y1
    h = max(1, y2 - y1 + 1)
    blocks.append((x, y, w, h))
blocks.sort()
l = 30 * MAX_L // (10**4)
for cy in range(1, 50):
    cx = 1
    for (x, y, w, h) in blocks:
        if cy in range(y, y + h):
            rs += dense_line(cy, x - cx, l, 0.99, cx)
            cx = x + w
    rs += dense_line(cy, MAX_L - 3 - cx, l, 0.99, cx)
rs += blocks
rs = fit(rs)
case(rs)

case_init(f"part_subdiv")
rs = scale_x(subdiv([(1, 1, 10**6, 49)], MAX_N))
case(rs)

# full
group_init()

case_init(f"sandwich2")
rs = fit(dense_line(2, MAX_L - 3, 21 * 10**4 // 11, 0.991), MAX_N - 2)
rs.append((MAX_L // 8, 1, MAX_L * 6 // 8, 1))
rs.append((1, 3, MAX_L - 3, 1))
case(scale(transpose(rs)))

case_init("full_checkers")
rs = scale(fit(checkerboard(1380, 1380, 0.8)))
case(rs)

case_init(f"dense_stripes2")
rs = scale(fit(dense_rect(100, MAX_L - 3, MAX_L  // 510, 0.99)))
case(rs)

case_init("subdiv1")
rs = scale(subdiv([(1, 1, MAX_L - 5, MAX_L - 5)], MAX_N))
case(rs)

case_init("subdiv2")
rs = scale(subdiv([(1, 1, 10**5, 10**5)], MAX_N))
case(rs)

case_init("simple_checkers")
rs = scale(fit(checkerboard(1150, 1150)))
case(rs)


assert not case_open
